<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 590 $
 * $Id: StationFactory.class.php 590 2007-01-31 22:29:44Z ipso $
 * $Date: 2007-01-31 14:29:44 -0800 (Wed, 31 Jan 2007) $
 */
include_once('Net/IPv4.php');
class StationFactory extends Factory {
	protected $table = 'station';
	protected $pk_sequence_name = 'station_id_seq'; //PK Sequence name

	protected $company_obj = NULL;

	protected $status_options = array(
											10 	=> 'DISABLED',
											20	=> 'ENABLED'
									);

	protected $type_options = array(
											10 	=> 'PC',
											20	=> 'PHONE',
											25	=> 'WirelessWeb',
											30 	=> 'iBUTTON',
											40 	=> 'Barcode',
											50 	=> 'FingerPrint',

									);

	protected $station_reserved_word_options = array('any', '*');
	protected $source_reserved_word_options = array('any', '*');

	protected $company_validator_msg = 'Company is invalid';
	protected $company_validator_label = 'company';

	protected $status_validator_options_msg = 'Incorrect Status';
	protected $status_validator_label = 'status';

	protected $type_validator_options_msg = 'Incorrect Type';
	protected $type_validator_label = 'type';

	protected $station_validator_length_msg = 'Incorrect Station ID length';
	protected $station_validator_unique_msg = 'Station ID already exists';
	protected $station_validator_label = 'station_id';

	protected $source_validator_length_msg = 'Incorrect Source ID length';
	protected $source_validator_label = 'source';

	protected $description_validator_length_msg = 'Incorrect Description length';
	protected $description_validator_label = 'description';

	protected $user_validator_msg = 'Incorrect Employee selection';
	protected $user_validator_label = 'user';

	function getCompanyObject() {
		if ( is_object($this->company_obj) ) {
			return $this->company_obj;
		} else {
			$clf = new CompanyListFactory();
			$this->company_obj = $clf->getById( $this->getCompany() )->getCurrent();

			return $this->company_obj;
		}
	}

	function getCompany() {
		return $this->data['company_id'];
	}
	function setCompany($id) {
		$id = trim($id);

		$clf = new CompanyListFactory();

		if ( $this->Validator->isResultSetWithRows(	$this->company_validator_label,
													$clf->getByID($id),
													$this->company_validator_msg
													) ) {

			$this->data['company_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getStatus() {
		//Have to return the KEY because it should always be a drop down box.
		//return Option::getByKey($this->data['status_id'], $this->getOptions('status') );
		return $this->data['status_id'];
	}
	function setStatus($status) {
		$status = trim($status);

		$key = Option::getByValue($status, $this->getOptions('status') );
		if ($key !== FALSE) {
			$status = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->status_validator_label,
											$status,
											$this->status_validator_options_msg,
											$this->getOptions('status')) ) {

			$this->data['status_id'] = $status;

			return FALSE;
		}

		return FALSE;
	}

	function getType() {
		//Have to return the KEY because it should always be a drop down box.
		//return Option::getByKey($this->data['status_id'], $this->getOptions('status') );
		return $this->data['type_id'];
	}
	function setType($type) {
		$type = trim($type);

		$key = Option::getByValue($type, $this->getOptions('type') );
		if ($key !== FALSE) {
			$type = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->type_validator_label,
											$type,
											$this->type_validator_options_msg,
											$this->getOptions('type')) ) {

			$this->data['type_id'] = $type;

			return FALSE;
		}

		return FALSE;
	}

	function isUniqueStation($station) {
		$ph = array(
					'company_id' => $this->getCompany(),
					'station' => $station,
					);

		$query = 'select id from '. $this->table .' where company_id = ? AND station_id = ? AND deleted=0';
		$id = $this->db->GetOne($query, $ph);
		Debug::Arr($id,'Unique Station: '. $station, __FILE__, __LINE__, __METHOD__,10);

		if ( $id === FALSE ) {
			return TRUE;
		} else {
			if ($id == $this->getId() ) {
				return TRUE;
			}
		}

		return FALSE;
	}

	function getStation() {
		if ( isset($this->data['station_id']) ) {
			return $this->data['station_id'];
		}

		return FALSE;
	}
	function setStation($station_id = NULL) {
		$station_id = trim($station_id);

		if ( empty($station_id) ) {
			$station_id = $this->genStationID();
		}

		if (	in_array(strtolower($station_id), $this->getOptions('station_reserved_word'))
				OR
				(
				$this->Validator->isLength(	$this->station_validator_label,
											$station_id,
											$this->station_validator_length_msg,
											2, 250 )
				AND
				$this->Validator->isTrue(	$this->station_validator_label,
											$this->isUniqueStation($station_id),
											$this->station_validator_unique_msg)
				)
			) {

			$this->data['station_id'] = $station_id;

			return FALSE;
		}

		return FALSE;
	}

	function getSource() {
		return $this->data['source'];
	}
	function setSource($source) {
		$source = trim($source);

		if ( 	in_array(strtolower($source), $this->getOptions('source_reserved_word') )
				OR
				(
				$source != NULL
				AND
				$this->Validator->isLength(	$this->source_validator_label,
											$source,
											$this->source_validator_length_msg,
											2, 250 )
				)
			) {

			$this->data['source'] = $source;

			return FALSE;
		}

		return FALSE;
	}

	function getDescription() {
		return $this->data['description'];
	}
	function setDescription($description) {
		$description = trim($description);

		if ( $this->Validator->isLength(	$this->description_validator_label,
											$description,
											$this->description_validator_length_msg,
											0, 255 ) ) {

			$this->data['description'] = $description;

			return FALSE;
		}

		return FALSE;
	}

	private function genStationID() {
		return md5( uniqid( dechex( mt_srand() ) ) );
	}

	function setCookie() {
		if ( $this->getStation() ) {

			setcookie('StationID', $this->getStation(), time()+999999999, Environment::getBaseURL() );

			return TRUE;
		}

		return FALSE;
	}

	function destroyCookie() {
		setcookie('StationID', NULL, time()+9999999, Environment::getBaseURL() );

		return TRUE;
	}

	function getUser() {
		$sulf = new StationUserListFactory();
		$sulf->getByStationId( $this->getId() );
		foreach ($sulf as $station_user) {
			$user_list[] = $station_user->getUser();
		}

		if ( isset($user_list) ) {
			return $user_list;
		}

		return FALSE;
	}
	function setUser($ids) {
		if (is_array($ids) and count($ids) > 0) {
			if ( in_array(0, $ids ) ) {
				Debug::text('All Users is selected: ', __FILE__, __LINE__, __METHOD__, 10);
				$ids = array(0);
			}

			//If needed, delete mappings first.
			$sulf = new StationUserListFactory();
			$sulf->getByStationId( $this->getId() );

			$user_ids = array();
			foreach ($sulf as $station_user) {
				$user_id = $station_user->getUser();
				Debug::text('Station ID: '. $station_user->getStation() .' User: '. $user_id, __FILE__, __LINE__, __METHOD__, 10);

				//Delete users that are not selected.
				if ( !in_array($user_id, $ids) ) {
					Debug::text('Deleting StationUser: '. $user_id, __FILE__, __LINE__, __METHOD__, 10);
					$station_user->Delete();
				} else {
					//Save branch ID's that need to be updated.
					Debug::text('NOT Deleting StationUser: '. $user_id, __FILE__, __LINE__, __METHOD__, 10);
					$user_ids[] = $user_id;
				}
			}

			//Insert new mappings.
			$suf = new StationUserFactory();
			foreach ($ids as $id) {
				if ( !in_array($id, $user_ids) ) {
					$suf->setStation( $this->getId() );
					$suf->setUser( $id );

					if ($this->Validator->isTrue(		$this->user_validator_label,
														$suf->Validator->isValid(),
														$this->user_validator_msg)) {
						$suf->save();
					}
				}
			}

			return TRUE;
		}

		return FALSE;
	}

	function getAllowedDate() {
		if ( isset($this->data['allowed_date']) ) {
			return $this->data['allowed_date'];
		}

		return FALSE;
	}
	function setAllowedDate($epoch = NULL) {
		$epoch = trim($epoch);

		if ($epoch == NULL) {
			$epoch = TTDate::getTime();
		}

		if 	(	$this->Validator->isDate(		$this->createddate_validator_label,
												$epoch,
												$this->date_validator_isdate_msg) ) {

			$this->data['allowed_date'] = $epoch;

			return TRUE;
		}

		return FALSE;

	}

	function checkSource( $source, $current_station_id ) {		
		$source = trim($source);
				
		$remote_addr = $_SERVER['REMOTE_ADDR'];
		if ( isset($_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
			$x_forwarded_for = $_SERVER['HTTP_X_FORWARDED_FOR'];
		} else {
			$x_forwarded_for = NULL;
		}
		
		//IGNORE x_forwarded_for for now, because anyone could spoof this.
		//Add a switch that will enable/disable this feature.
		
		//$remote_addr = '192.168.2.10';
		//$remote_addr = '192.168.1.10';
		//$remote_addr = '127.0.0.1';
		
		if ( in_array( $this->getType(), array(10,25) ) AND preg_match('/[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}(\/[0-9]{1,2})*/', $source) ) {
			Debug::text('Source is an IP address!', __FILE__, __LINE__, __METHOD__, 10);
		} elseif ( in_array( $this->getType(), array(10,25) ) )  {
			Debug::text('Source is NOT an IP address, do hostname lookup: '. $source , __FILE__, __LINE__, __METHOD__, 10);

			$hostname_lookup = $this->getCache( $remote_addr.$source );
			if ( $hostname_lookup === FALSE ) {
				$hostname_lookup = gethostbyname( $source );

				$this->saveCache($hostname_lookup, $remote_addr.$source );
			}

			if ($hostname_lookup == $source ) {
				Debug::text('Hostname lookup failed!', __FILE__, __LINE__, __METHOD__, 10);
			} else {
				Debug::text('Hostname lookup succeeded: '. $hostname_lookup, __FILE__, __LINE__, __METHOD__, 10);
				$source = $hostname_lookup;
			}
			unset($hostname_lookup);
		} else {
			Debug::text('Source is not internet related', __FILE__, __LINE__, __METHOD__, 10);
		}
		
		Debug::text('Source: '. $source .' Remote IP: '. $remote_addr .' Behind Proxy IP: '. $x_forwarded_for, __FILE__, __LINE__, __METHOD__, 10);		
		if ( 	(
					$current_station_id == $this->getStation()
						OR in_array( strtolower( $this->getStation() ), $this->getOptions('station_reserved_word') )
				)
				AND
				(
					in_array( strtolower( $this->getSource() ), $this->getOptions('source_reserved_word') )
					OR
						( $source == $remote_addr
							/*OR $source == $x_forwarded_for*/ )
					OR
						( $current_station_id == $this->getSource() )
					OR
						( Net_IPv4::ipInNetwork( $remote_addr, $source) )
				)

			) {

			Debug::text('Returning TRUE', __FILE__, __LINE__, __METHOD__, 10);
				
			return TRUE;
		}
		
		Debug::text('Returning FALSE', __FILE__, __LINE__, __METHOD__, 10);
		return FALSE;
	}
	
	function isAllowed($user_id = NULL, $current_station_id = NULL, $id = NULL) {
		if ($user_id == NULL OR $user_id == '') {
			global $current_user;
			$user_id = $current_user->getId();
		}
		Debug::text('User ID: '. $user_id, __FILE__, __LINE__, __METHOD__, 10);

		if ($current_station_id == NULL OR $current_station_id == ''){
			global $current_station;
			$current_station_id = $current_station->getStation();
		}
		Debug::text('Station ID: '. $current_station_id, __FILE__, __LINE__, __METHOD__, 10);

		Debug::text('Status: '. $this->getStatus(), __FILE__, __LINE__, __METHOD__, 10);
		if ( $this->getStatus() != 20 ) { //Enabled
			return FALSE;
		}

		$retval = FALSE;
		
		$user_list = $this->getUser();

		if ( is_array($user_list) AND ( in_array( -1, $user_list) OR in_array($user_id, $user_list) ) ) {
			Debug::text('User ID in list!: ', __FILE__, __LINE__, __METHOD__, 10);
			Debug::text('Current Station: '. $this->getStation() , __FILE__, __LINE__, __METHOD__, 10);

			//Handle IP Addresses/Hostnames
			if ( $this->getType() == 10
					AND !in_array( strtolower( $this->getSource() ), $this->getOptions('source_reserved_word') ) ) {
				
				if ( strpos( $this->getSource(), ',') !== FALSE ) {
					//Found list
					$source = explode(',', $this->getSource() );
				} else {
					//Found single entry
					$source[] = $this->getSource();
				}
				
				if ( is_array($source) ) {
					foreach( $source as $tmp_source ) {
						if ( $this->checkSource( $tmp_source, $current_station_id ) == TRUE ) {
							$retval = TRUE;
							break;
						}
					}
					unset($tmp_source);
				}				
			} else {
				$source = $this->getSource();
				
				$retval = $this->checkSource( $source, $current_station_id );
			}

			Debug::text('Retval: '. $retval, __FILE__, __LINE__, __METHOD__, 10);
			//Debug::text('Current Station ID: '. $current_station_id .' Station ID: '. $this->getStation(), __FILE__, __LINE__, __METHOD__, 10);
			
			if ( $retval === TRUE ) {
				Debug::text('Station IS allowed! ', __FILE__, __LINE__, __METHOD__, 10);

				//Set last allowed date, so we can track active/inactive stations.
				if ( $id != NULL AND $id != '' ) {
					$slf = new StationListFactory();
					$slf->getById( $id );
					foreach ($slf as $station_obj) {
						$station_obj->setAllowedDate();
						$station_obj->Save();
						Debug::text('Updating last allowed time for station.', __FILE__, __LINE__, __METHOD__, 10);
					}
					unset($slf, $station_obj);

					TTLog::addEntry( $id, 'Allow', ' Access from station Allowed ', $user_id, $this->getTable() );
				}

				return TRUE;
			}
		}

		Debug::text('Station IS NOT allowed! ', __FILE__, __LINE__, __METHOD__, 10);

		return FALSE;
	}

	//A fast way to check many stations if the user is allowed.
	function checkAllowed($user_id = NULL, $station_id = NULL, $type = 'PC') {
		if ($user_id == NULL OR $user_id == '') {
			global $current_user;
			$user_id = $current_user->getId();
		}
		Debug::text('User ID: '. $user_id, __FILE__, __LINE__, __METHOD__, 10);

		if ($station_id == NULL OR $station_id == ''){
			global $current_station;
			if ( is_object($current_station) ) {
				$station_id = $current_station->getStation();
			} else {
				Debug::text('Unable to get Station Object! Station ID: '. $station_id, __FILE__, __LINE__, __METHOD__, 10);
				return FALSE;
			}
		}
		Debug::text('Station ID: '. $station_id, __FILE__, __LINE__, __METHOD__, 10);

		$slf = new StationListFactory();
		$slf->getByUserIdAndStatusAndType($user_id, 'ENABLED', $type);
		foreach($slf as $station) {
			Debug::text('Checking Station ID: '. $station->getId(), __FILE__, __LINE__, __METHOD__, 10);

			if ( $station->isAllowed( $user_id, $station_id, $station->getId() ) === TRUE) {
				Debug::text('Station IS allowed! '. $station_id .' - ID: '. $station->getId() , __FILE__, __LINE__, __METHOD__, 10);
				return TRUE;
			}
		}

		return FALSE;
	}

	function postSave() {
		$this->removeCache( $this->getStation() );
/*
		foreach ($this->getUser() as $user_id ) {
			$cache_id = 'station::checkAllowed_'.$this->getId().$user_id;
			$this->removeCache( $cache_id );
		}
*/
		return TRUE;
	}

	function addLog( $log_action ) {
		return TTLog::addEntry( $this->getId(), $log_action, 'Station', NULL, $this->getTable() );
	}
}
?>
