<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 497 $
 * $Id: UserWageFactory.class.php 497 2007-01-05 00:39:27Z ipso $
 * $Date: 2007-01-04 16:39:27 -0800 (Thu, 04 Jan 2007) $
 */
class UserWageFactory extends Factory {
	protected $table = 'user_wage';
	protected $pk_sequence_name = 'user_wage_id_seq'; //PK Sequence name

	var $user_obj = NULL;
	var $labor_standard_obj = NULL;
	var $holiday_obj = NULL;

	protected $type_options = array(
											10 	=> 'Hourly',
											20	=> 'Salary (Annual)',
//											30	=> 'Min. Wage + Bonus (Salary)'
									);

	protected $type_validator_options_msg = 'Incorrect Type';
	protected $type_validator_label = 'type';

	protected $user_validator_msg = 'Invalid User';
	protected $user_validator_label = 'user';

	protected $wage_validator_isfloat_msg = 'Incorrect Wage';
	protected $wage_validator_label = 'wage';

	protected $weeklytime_validator_isfloat_msg = 'Incorrect Weekly Time';
	protected $weeklytime_validator_label = 'weekly_time';

	protected $laborburdenpercent_validator_isfloat_msg = 'Incorrect Labor Burden Percent';
	protected $laborburdenpercent_validator_label = 'labor_burden_percent';

	protected $effectivedate_validator_isdate_msg = 'Incorrect Effective Date';
	protected $effectivedate_validator_msg = 'Effective Date must not be in the middle of a pay period';
	protected $effectivedate_validator_label = 'effective_date';

	protected $note_validator_length_msg = 'Note is too long';
	protected $note_validator_label = 'note';

	function getUserObject() {
		if ( is_object($this->user_obj) ) {
			return $this->user_obj;
		} else {
			$ulf = new UserListFactory();
			$this->user_obj = $ulf->getById( $this->getUser() )->getCurrent();

			return $this->user_obj;
		}
	}

	function getUser() {
		return $this->data['user_id'];
	}
	function setUser($id) {
		$id = trim($id);

		$ulf = new UserListFactory();

		if ( $id == 0
				OR $this->Validator->isResultSetWithRows(	$this->user_validator_label,
															$ulf->getByID($id),
															$this->user_validator_msg
															) ) {
			$this->data['user_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getType() {
		if ( isset($this->data['type_id']) ) {
			return $this->data['type_id'];
		}
		
		return FALSE;
	}
	function setType($type) {
		$type = trim($type);

		$key = Option::getByValue($type, $this->getOptions('type') );
		if ($key !== FALSE) {
			$type = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->type_validator_label,
											$type,
											$this->type_validator_options_msg,
											$this->getOptions('type')) ) {

			$this->data['type_id'] = $type;

			return TRUE;
		}

		return FALSE;
	}

	function getWage() {
		if ( isset($this->data['wage']) ) {
			return $this->data['wage'];
		}
		
		return FALSE;
	}
	function setWage($wage) {
		$wage = trim($wage);

		//Pull out only digits and periods.
		$wage = $this->Validator->stripNonFloat($wage);

		if (	$this->Validator->isFloat(	$this->wage_validator_label,
											$wage,
											$this->wage_validator_isfloat_msg) ) {

			$this->data['wage'] = $wage;

			return TRUE;
		}

		return FALSE;
	}

	function getWeeklyTime() {
		if ( isset($this->data['weekly_time']) ) {
			//Debug::Text('Weekly Time: '. $this->data['weekly_time'], __FILE__, __LINE__, __METHOD__,10);

			return $this->data['weekly_time'];
		}

		return FALSE;

	}
	function setWeeklyTime($value) {
		//$value = $value;

		if (	$value == NULL
				OR
				$this->Validator->isNumeric(	$this->weeklytime_validator_label,
											$value,
											$this->weeklytime_validator_isfloat_msg) ) {

			$this->data['weekly_time'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getLaborBurdenPercent() {
		if ( isset($this->data['labor_burden_percent']) ) {
			return $this->data['labor_burden_percent'];
		}
		
		return FALSE;
	}
	function setLaborBurdenPercent($value) {
		$value = trim($value);

		//Pull out only digits and periods.
		$value = $this->Validator->stripNonFloat($value);

		if (	$this->Validator->isFloat(	$this->laborburdenpercent_validator_label,
											$value,
											$this->laborburdenpercent_validator_isfloat_msg) ) {

			$this->data['labor_burden_percent'] = $value;

			return TRUE;
		}

		return FALSE;
	}


	function isValidEffectiveDate($epoch) {
		return TRUE;
	}

	function getEffectiveDate( $raw = FALSE ) {
		if ( isset($this->data['effective_date']) ) {
			if ( $raw === TRUE ) {
				return $this->data['effective_date'];
			} else {
				return TTDate::strtotime( $this->data['effective_date'] );
			}
		}

		return FALSE;
	}
	function setEffectiveDate($epoch) {
		$epoch = TTDate::getBeginDayEpoch( trim($epoch) );

		Debug::Text('Effective Date: '. TTDate::getDate('DATE+TIME', $epoch ) , __FILE__, __LINE__, __METHOD__,10);

		if 	(	$this->Validator->isDate(		$this->effectivedate_validator_label,
												$epoch,
												$this->effectivedate_validator_isdate_msg)
				AND
				$this->Validator->isTrue(		$this->effectivedate_validator_label,
												$this->isValidEffectiveDate($epoch),
												$this->effectivedate_validator_msg)
			) {

			$this->data['effective_date'] = $epoch;

			return TRUE;
		}

		return FALSE;
	}

	function getNote() {
		if ( isset($this->data['note']) ) {
			return $this->data['note'];
		}

		return FALSE;
	}
	function setNote($value) {
		$value = trim($value);

		if (	$value == ''
				OR
						$this->Validator->isLength(		$this->note_validator_label,
														$value,
														$this->note_validator_length_msg,
														1,
														2048)
			) {

			$this->data['note'] = $value;

			return FALSE;
		}

		return FALSE;
	}

	function getLaborBurdenHourlyRate( $exclude_stats = FALSE, $exclude_vacation = FALSE, $epoch = FALSE ) {
		$retval = bcmul( $this->getHourlyRate( $exclude_stats , $exclude_vacation, $epoch ), bcadd( bcdiv( $this->getLaborBurdenPercent(), 100 ), 1) );
		
		return Misc::MoneyFormat($retval, FALSE);
	}
	
	function getHourlyRate( $exclude_stats = FALSE, $exclude_vacation = FALSE, $epoch = FALSE ) {
		$hourly_wage = 0;

		switch ( $this->getType() ) {
			case 10: //Hourly
				//Debug::text('Hourly Wage' , __FILE__, __LINE__, __METHOD__,10);
				$hourly_wage = $this->getWage();
				break;
			case 20: //Salary
				//FIXME: make sure this is correct. With stat days/vacation days stuff.
				$hourly_wage = $this->getAnnualHourlyRate( $exclude_stats, $exclude_vacation, $epoch );
				break;
		}

		return Misc::MoneyFormat($hourly_wage, FALSE);
	}

	function getAnnualHourlyRate( $exclude_stats = FALSE, $exclude_vacation = FALSE, $epoch = FALSE ) {
		if ( $epoch == FALSE ) {
			$epoch = TTDate::getTime();
		}
		//Debug::text('EPOCH: '. $epoch , __FILE__, __LINE__, __METHOD__,10);


		//Should not use week days here? Since some staff might work weekends?
		$annual_week_days = TTDate::getAnnualWeekDays( $epoch );
		//Debug::text('Annual Week Days: '. $annual_week_days , __FILE__, __LINE__, __METHOD__,10);

		//Calculate weeks from adjusted annual weekdays
		$annual_work_weeks = bcdiv( $annual_week_days, 5);
		//Debug::text('Adjusted annual work weeks : '. $annual_work_weeks , __FILE__, __LINE__, __METHOD__,10);

		$average_weekly_hours = TTDate::getHours( $this->getWeeklyTime() );
		//Debug::text('Average Weekly Hours: '. $average_weekly_hours , __FILE__, __LINE__, __METHOD__,10);

		if ( $average_weekly_hours == 0 ) {
			//No default schedule, can't pay them.
			$hourly_wage = 0;
		} else {
			//Divide by average hours/day from default schedule?
			//$hourly_wage = @( $this->getWage() / ( $annual_work_weeks * $average_weekly_hours ) );
			$hours_per_year = bcmul($annual_work_weeks, $average_weekly_hours);
			if ( $hours_per_year > 0 ) {
				//$hourly_wage = @( $this->getWage() / ( $annual_work_weeks * $average_weekly_hours ) );
				$hourly_wage = bcdiv( $this->getWage(), $hours_per_year );
			}
			unset($hours_per_year);
		}
		//Debug::text('User Wage: '. $this->getWage() , __FILE__, __LINE__, __METHOD__,10);
		//Debug::text('Annual Hourly Rate: '. $hourly_wage , __FILE__, __LINE__, __METHOD__,10);

		return $hourly_wage;
	}

	static function proRateSalary($salary, $wage_effective_date, $prev_wage_effective_date, $pp_start_date, $pp_end_date ) {
		$prev_wage_effective_date = (int)$prev_wage_effective_date;

		if ( $wage_effective_date < $pp_start_date ) {
			$wage_effective_date = $pp_start_date;
		}

		$total_pay_period_days = ceil( TTDate::getDayDifference( $pp_start_date, $pp_end_date) );

		if ( $prev_wage_effective_date == 0 ) {
			Debug::text(' Using Pay Period End Date: '. TTDate::GetDate('DATE', $pp_end_date) , __FILE__, __LINE__, __METHOD__,10);
			$total_wage_effective_days = ceil( TTDate::getDayDifference( $wage_effective_date, $pp_end_date) );
		} else {
			Debug::text(' Using Prev Effective Date: '. TTDate::GetDate('DATE', $prev_wage_effective_date ) , __FILE__, __LINE__, __METHOD__,10);
			$total_wage_effective_days = ceil( TTDate::getDayDifference( $wage_effective_date, $prev_wage_effective_date ) );
		}

		Debug::text('Salary: '. $salary .' Total Pay Period Days: '. $total_pay_period_days .' Wage Effective Days: '. $total_wage_effective_days , __FILE__, __LINE__, __METHOD__,10);

		//$pro_rate_salary = $salary * ($total_wage_effective_days / $total_pay_period_days);
		$pro_rate_salary = bcmul( $salary, bcdiv($total_wage_effective_days, $total_pay_period_days) );

		Debug::text('Pro Rate Salary: '. $pro_rate_salary, __FILE__, __LINE__, __METHOD__,10);
		return $pro_rate_salary;
	}

	static function getWageFromArray( $date, $wage_arr ) {
		if ( !is_array($wage_arr) ) {
			return FALSE;
		}

		if ( $date == '' ) {
			return FALSE;
		}

		//Debug::Arr($wage_arr, 'Wage Array: ', __FILE__, __LINE__, __METHOD__,10);

		foreach( $wage_arr as $effective_date => $wage ) {
			if ( $effective_date <= $date ) {
				Debug::Text('Effective Date: '. TTDate::getDate('DATE+TIME', $effective_date) .' Is Less Than: '. TTDate::getDate('DATE+TIME', $date)  , __FILE__, __LINE__, __METHOD__,10);
				return $wage;
			}
		}

		return FALSE;
	}
	
	//Takes the employees 
	static function calculateLaborBurdenPercent( $company_id, $user_id ) {
		if ( $company_id == '' ) {
			return FALSE;		
		}
		if ( $user_id == '' ) {
			return FALSE;		
		}
		
		$end_epoch = TTDate::getTime();
		$start_epoch = TTDate::getTime()-(86400*180); //6mths
				
		$retval = FALSE;
		
		$pseallf = new PayStubEntryAccountLinkListFactory();
		$pseallf->getByCompanyID( $company_id );
		if ( $pseallf->getRecordCount() > 0 ) {			
			$pself = new PayStubEntryListFactory();
			$total_gross = $pself->getAmountSumByUserIdAndEntryNameIdAndStartDateAndEndDate($user_id, $pseallf->getCurrent()->getTotalGross(), $start_epoch, $end_epoch );
			$total_employer_deductions = $pself->getAmountSumByUserIdAndEntryNameIdAndStartDateAndEndDate($user_id, $pseallf->getCurrent()->getTotalEmployerDeduction(), $start_epoch, $end_epoch );
			
			if ( isset($total_employer_deductions['amount']) AND isset($total_gross['amount']) ) {
				$retval = bcmul( bcdiv( $total_employer_deductions['amount'], $total_gross['amount']), 100, 2);
			}
		}
		
		return $retval;
	}

	function preSave() {
		if ( $this->getType() == 10 ) { //Hourly
			$this->setWeeklyTime( NULL );
		}

		return TRUE;
	}

	function postSave() {
		$this->removeCache( $this->getId() );

		return TRUE;
	}

	function addLog( $log_action ) {
		return TTLog::addEntry( $this->getId(), $log_action, 'User Wage', NULL, $this->getTable() );
	}
}
?>
